# frozen_string_literal: true

# Copyright from 2019 Paulo Arruda (parrudaj at gmail dot com)
#
# Licensed under the Commons Clause + Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

STDOUT.sync = true
Process.setproctitle('ruby_runner')
GC.enable
require 'erb'
require 'etc'
require 'base64'
require 'yaml'
require 'socket'
require 'timeout'
require 'oj'
# require 'faastruby-rpc'
require 'msgpack'

require 'gotta/run/ruby/runtime/errors'
require 'gotta/run/ruby/loader/executor'
require 'gotta/run/ruby/loader/component'
require 'gotta/run/ruby/runtime/event'
require 'gotta/run/ruby/runtime/response'
require 'gotta/run/ruby/runtime/template'
require 'gotta/run/ruby/runtime'

# require 'redis'
# require 'ohm'
# require 'jwt'
# require 'active_support'
# require 'active_record'
# require 'mysql2'
# require 'pg'

Object.send(:remove_const, :TOPLEVEL_BINDING)
MAX_EXECUTION_TIME = ENV['MAX_EXECUTION_TIME'].to_i
PROCESSORS = Etc.nprocessors
Oj.default_options = { mode: :strict, symbol_keys: false }

loop do
  pid = fork do
    Module.new do
      include Gotta::Run::Ruby::Runtime

      socket = UNIXSocket.for_fd(ENV['SOCKET_FD'].to_i)

      # puts 'Waiting for message'
      begin
        payload = MessagePack.unpack(socket.recv(10_485_760))
      rescue Interrupt
        exit 0
      end
      # puts "Got message! #{payload}"
      event = self::Event.new(payload['event'])

      unless set_path(payload['load_path'])
        socket.send(
          self::Response.not_found(
            payload['load_path']
          ).to_msgpack,
          0
        )
        exit 1
      end

      start_time, cpu_start_time = get_time

      run_component(event)

      cpu_end_time, end_time = get_time

      stats_hash = build_stats(
        calculate_times(start_time, end_time),
        calculate_times(cpu_start_time, cpu_end_time)
      )

      payload = if response.is_a?(Response)
                  response.with_stats(stats_hash)
                else
                  self::Response.new(
                    body: {
                      error: "Your function must render a response. For example, render text: \"Hello World!\". If you want to respond with an empty body, use 'render_nothing'."
                    },
                    status: 422,
                    headers: { 'Content-Type' => 'application/json' },
                    stats: stats_hash
                  )
                end

      socket.send payload.to_msgpack, 0
      exit 0
    end
  end
  begin
    Process.wait(pid)
  rescue Interrupt
    exit 0
  end
  sleep 1
end
